/*
  pinctrl.h - GPIO Control Library for Linux GPIO Character Device

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <stdint.h>
#include <stdbool.h>
#include <linux/gpio.h>
#include "pinmode.h"

#ifndef PINCTRL_MAX_CHIPS
#define PINCTRL_MAX_CHIPS 4
#endif

#ifndef PINCTRL_MAX_EVENTS
#define PINCTRL_MAX_EVENTS 16
#endif

#define PINCTRL_MAX_LINES (PINCTRL_MAX_CHIPS * GPIO_V2_LINES_MAX)

typedef void (*pinctrl_callback_t)(void *param, const struct gpio_v2_line_event& event);

class PinCtrl
{
  public:

    PinCtrl(void);
    virtual ~PinCtrl(void);
    size_t begin(void);
    const struct gpiochip_info *chip_info(size_t num);
    void end(void);
    void release(pin_size_t pin);
    void stop(pin_size_t pin);
    void pinMode(pin_size_t pin, PinMode mode, PinPull pull = PULL_DISABLE, uint32_t debounce_period_us = 0);
    void digitalWrite(pin_size_t pin, PinStatus value);
    PinStatus digitalRead(pin_size_t pin);
    void attachInterruptParam(pin_size_t pin, pinctrl_callback_t func, void *arg, PinEdge edge, PinPull pull = PULL_DISABLE, uint32_t debounce_period_us = 0);
    void detachInterrupt(pin_size_t pin);
    void sleepMicroseconds(size_t usec);
    void sleep(size_t msec);
    void delayMicroseconds(size_t usec);
    void delay(size_t msec);
    size_t micros(void);
    size_t millis(void);

  protected:

    int config(pin_size_t pin, gpio_v2_line_config& cfg);

  private:

    size_t               _chip_count;
    struct gpiochip_info _chip_info[PINCTRL_MAX_CHIPS];
    int                  _chip_fd[PINCTRL_MAX_CHIPS];
    int                  _line_fd[PINCTRL_MAX_LINES];
    int                  _pipe_fd[PINCTRL_MAX_LINES];
};

extern PinCtrl GPIO;
